rm(list=ls())
library(missForest)
library(mvtnorm)
library(missMDA)
library(AER)
library(ipred)
setwd("D:/Documents and Settings/PRI.MATH0957/Bureau/Audigier_Donnees_Manquantes/Article_Recherche/Script_R/Programmes_apres_reviewing/Programmes_en_ligne/")
source("MonmissForest.r")
source("AFDM_RF.r")
Sim<-200

################
#section 3.1 (Relationships between continuous and categorical variables)
################
  #creating a data set
  V<-matrix(0,ncol=8,nrow=8)
  V[1:4,1:4]<-1
  V[5:8,5:8]<-1
  Jeu<-Simulation(V=V,quanti=c(1,3,5,7),mod=rep(4,4),taille=150,sigma.quanti=0.26,sigma.quali=0.017)

  #Performing algorithms on the data set, as well as on continuous variables only, and on categorical variables only
  Comp.sim.ter<-Comparaison(Jeu=Jeu[[2]],Jeu.nb=Jeu[[1]],borne=7,Sim=Sim,RF=F)
  Comp.sim.quanti.ter<-Comparaison(Jeu=Jeu[[2]][,seq(1,8,2)],Jeu.nb=Jeu[[1]][,seq(1,8,2)],borne=2, Sim=Sim,RF=F)
  Comp.sim.quali.ter<-Comparaison(Jeu=Jeu[[2]][,-seq(1,8,2)],Jeu.nb=Jeu[[1]][,-seq(1,8,2)],borne=6, Sim=Sim,RF=F)

  #extracting results
  err.mixte<-Analyse3.1(Comp.sim.ter,ncp=7)
  err.quanti<-Analyse3.1(Comp.sim.quanti.ter,ncp=2)
  err.quali<-Analyse3.1(Comp.sim.quali.ter,ncp=6)

  #FIGURE 2
  par(cex=0.9,mar = c(2.1, 2.1, 2.1, 2.1))
  boxplot(err.quanti[[1]][,1],err.mixte[[1]][[1]],err.quanti[[1]][,2],err.mixte[[1]][[2]],err.quanti[[1]][,3],err.mixte[[1]][[3]],names=c("10%","10%","20%","20%","30%","30%"),ylab = "NRMSE",col=rep(c(gray(.5),gray(.75)),3),ylim=echelle(data.frame(err.quanti[[1]][,1],err.mixte[[1]][[1]],err.quanti[[1]][,2],err.mixte[[1]][[2]],err.quanti[[1]][,3],err.mixte[[1]][[3]])))
  legend("topleft",c("Whitout categorical variables","With categorical variables"),col=c(gray(.5),gray(.75)),pch=15)   

  #FIGURE 3
  par(cex=0.9,mar = c(2.1, 2.1, 2.1, 2.1))
  boxplot(err.quali[[1]][,1],err.mixte[[2]][[1]],err.quali[[1]][,2],err.mixte[[2]][[2]],err.quali[[1]][,3],err.mixte[[2]][[3]],names=c("10%","10%","20%","20%","30%","30%"),ylab = "PFC",col=rep(c(gray(.9),gray(.75)),3),ylim=echelle(data.frame(err.quali[[1]][,1],err.mixte[[2]][[1]],err.quali[[1]][,2],err.mixte[[2]][[2]],err.quali[[1]][,3],err.mixte[[2]][[3]])))
  legend("topleft",c("Without continuous variables","With continuous variables"),col=c(gray(.9),gray(.75)),pch=15)   

################
# section 3.2.1 (Linear and nonlinear relationships)
################

  #FIGURE 4

  #generating data
  taille<-100
  V<-matrix(1,ncol=5,nrow=5)
  quanti<-c(1,2)
  mod<-c(4,4,4)
  ncp<-4
  tol<-0.05
  Jeu<-Simulation(V=V,quanti=quanti,mod=mod,taille=taille,sigma.quanti=0.58,sigma.quali=0.033)

  #performing simulations
  Comp<-Comparaison(Jeu[[2]],Jeu[[1]],borne=ncp,Sim=Sim)

  #drawing errors
  Analyse(Comp,ncp=3)


  #FIGURE 5
  taille<-200

    #linear

    #genarating data
    borne<-4
    V<-matrix(1,4,4)
    Jeu<-Simulation(V,quanti=c(1:3),mod=c(10),taille=taille)
    #performing simulations
    Comp.lin.mixte<-Comparaison(Jeu=Jeu[[2]],Jeu.nb=Jeu[[1]],Sim=Sim,borne=borne)
    names(Comp.lin.mixte)<-NULL
    #plotting results
    Analyse(Comp.lin.mixte,ncp=4,crit.aff=1)

    #nonlinear
    
    #genarating data
    Jeu[[1]][,2]<-(Jeu[[1]][,2])^2
    Jeu[[2]][,2]<-(Jeu[[2]][,2])^2
    Jeu[[1]][,3]<-cos(Jeu[[1]][,3]*2)
    Jeu[[2]][,3]<-cos(Jeu[[2]][,3]*2)
    #performing simulations
    Comp.nonlin.mixte<-Comparaison(Jeu=Jeu[[2]],Jeu.nb=Jeu[[1]],Sim=Sim,borne=borne)
    names(Comp.nonlin.mixte)<-NULL
    #plotting results
    Analyse(Comp.nonlin.mixte,ncp=4,crit.aff=1)

################
# section 3.2.2 (Taking into account interactions between categorical variables)
################
  # FIGURE 7
  #genarating data
  m<-matrix(c(rep(c("a","b","c"),3),c(rep("a",3),rep("b",3),rep("c",3)),c(1,2,3,2,3,1,3,1,2)),ncol=3)
  M<-c()
  for (i in 1:(ceiling(taille/9))){M<-rbind(M,m)}
  M<-as.data.frame(M)
  M[,3]<-as.numeric(M[,3])
  #performing simulations
  Comp.acinter<-Comparaison(Jeu=M,Jeu.nb=M,Sim=Sim,borne=1)
  names(Comp.acinter)<-NULL
  #plotting results
  Analyse(Comp.acinter,crit.aff=1,ncp=1)
  Analyse(Comp.acinter,crit.aff=2,ncp=1)

################
# section 3.4 (cf Entensive study.R)
################

################
# section 3.5 (Choice of the number of dimensions)
################
  # FIGURE 8

  taille<-150
  V<-matrix(0,12,12)
  V[1:8,1:8]<-1
  V[9:12,9:12]<-1
  quanti<-seq(1,12,2)
  mod<-rep(3,6)

    #generating data, performing simulations and plotting results with snr=1
    Jeu<-Simulation(V=V,quanti=quanti,mod=mod,taille=taille,sigma.quanti=2.2,sigma.quali=1.097)
    Comp_sim_snr1<-Comparaison(Jeu=Jeu[[2]],Jeu.nb=Jeu[[1]],Sim=Sim,borne=6, RF=F)
    Err.nbaxe(Comp_sim_snr1)
    graphics.off()

    #generating data, performing simulations and plotting results with snr=3
    Jeu<-Simulation(V=V,quanti=quanti,mod=mod,taille=taille,sigma.quanti=.22,sigma.quali=.017)
    Comp_sim_snr3<-Comparaison(Jeu=Jeu[[2]],Jeu.nb=Jeu[[1]],Sim=Sim,borne=6,RF=F)
    Err.nbaxe(Comp_sim_snr3)

################
# section 4 (Real data sets)
################
  #FIGURE 9

  ## Tips
  #importing data
  source("http://www.info.univ-angers.fr/~gh/statgh.r")
  Tips <- lit.dar("tips.dar")
  Tips[["DAY"]]<-as.factor(Tips[["DAY"]])
  Tips[["SEX"]]<-as.factor(Tips[["SEX"]])
  Tips[["SMOKER"]]<-as.factor(Tips[["SMOKER"]])
  Tips[["TIME"]]<-as.factor(Tips[["TIME"]])
  Tips[,c(1:2,7)]<-scale(Tips[,c(1:2,7)])
  #performing simulations
  Comp.Tips<-Comparaison(Tips,name.jeu=c("Tips"),borne=3,Sim=Sim)
  Comp.tips<-list(Comp.Tips[["Tips"]][-c(88,75,77)])
  names(Comp.tips)<-NULL
  #plotting results
  Analyse(Comp.tips,ncp=3,crit.aff=1)
  Analyse(Comp.tips,ncp=3,crit.aff=2)

  ##BMI
  Imc<-read.table("http://www.biostatisticien.eu/springeR/imcenfant.txt",header=T)
  Imc[,3:6]<-scale(Imc[,3:6])
  Comp.Imc<-Comparaison(Imc,name.jeu=c("Imc"),borne=2,Sim=Sim)
  names(Comp.Imc)<-NULL
  Analyse(Comp.Imc,ncp=2,crit.aff=1)
  Analyse(Comp.Imc,ncp=2,crit.aff=2)

  ## GBSG2
  data(GBSG2)
  Comp.GBSG2<-Comparaison(GBSG2,borne=2,Sim=Sim,name.jeu=c("GBSG2"))
  names(Comp.GBSG2)<-NULL
  Analyse(Comp.GBSG2,ncp=2,crit.aff=1)
  Analyse(Comp.GBSG2,ncp=2,crit.aff=2)

  ## Ozone
  Ozone<-read.table("http://www.agrocampus-ouest.fr/math/livreR/ozone.txt",sep=" ",header=T)
  Comp.Ozone<-Comparaison(Ozone,borne=2,Sim=Sim,name.jeu=c("Ozone"))
  names(Comp.Ozone)<-NULL
  Analyse(Comp.Ozone,ncp=2,crit.aff=1)
  Analyse(Comp.Ozone,ncp=2,crit.aff=2)

  #FIGURE 10
  ## Parkinson
  Parkinson <- read.table("Parkinsons.data",sep=",",header=T,row.names=1)[,-17]
  Comp.Parkinson<-Comparaison(Parkinson,borne=5,Sim=Sim,name.jeu=c("Parkinson"))
  names(Comp.Parkinson)<-NULL
  Analyse(Comp.Parkinson,ncp=5)

  ##Credit
  set.seed(0)
  Credit<-read.csv("credit.csv",header=T,sep=";")
  Credit<-Credit[-which(Credit[,1]=="Side-car"),]
  levels(Credit[,1])[5]<-levels(Credit[,1])[1]
  Credit[,ncol(Credit)]<-as.factor(Credit[,ncol(Credit)])
  Comp.Credit<-Comparaison(Credit,borne=3,Sim=250,name.jeu=c("Credit"))
  Comp<-list(Comp.Credit[["Credit"]][-c(20,22,23,24,27,29,30,34,38,45,48,51,55,61,62,64,65,68,72,76,78,96,107,108,115,120,122,123,124,126,132,134,139,142,152,153,155,157,160,161,162,171,178,181,182,185,194,197,213,215,216,220,226,229:232,247:248)])
  names(Comp)<-NULL
  Analyse(Comp,ncp=3)